@@ -16,7 +16,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::QuickTime;
 
-$VERSION = '1.02';
+$VERSION = '1.03';
 
 sub ProcessGoPro($$$);
 sub ProcessString($$$);
@@ -65,20 +65,20 @@ my %addUnits = (
     PROCESS_PROC => \&ProcessGoPro,
     GROUPS => { 2 => 'Camera' },
     NOTES => q{
-        Tags extracted from the GPMF box of GoPro MP4 videos, the APP6 "GoPro" segment
-        of JPEG files, and from the "gpmd" timed metadata if the ExtractEmbedded option
-        is enabled.  Many more tags exist, but are currently unknown and extracted only
-        with the -u option. Please let me know if you discover the meaning of any of
-        these unknown tags. See L<https://github.com/gopro/gpmf-parser> for details
-        about this format.
+        Tags extracted from the GPMF box of GoPro MP4 videos, the APP6 "GoPro"
+        segment of JPEG files, and from the "gpmd" timed metadata if the
+        L<ExtractEmbedded|../ExifTool.html#ExtractEmbedded> (-ee) option is enabled.  Many more tags exist, but are
+        currently unknown and extracted only with the L<Unknown|../ExifTool.html#Unknown> (-u) option. Please
+        let me know if you discover the meaning of any of these unknown tags. See
+        L<https://github.com/gopro/gpmf-parser> for details about this format.
     },
     ACCL => { #2 (gpmd)
         Name => 'Accelerometer',
-        Notes => 'accelerator readings in m/s',
+        Notes => 'accelerator readings in m/s2',
         Binary => 1,
     },
     ALLD => 'AutoLowLightDuration', #1 (gpmd) (untested)
-  # APTO (GPMF) - seen: 'RAW' (fmt c)
+  # APTO (GPMF) - seen: 'RAW', 'DYNM' (fmt c)
     ATTD => { #PH (Karma)
         Name => 'Attitude',
         # UNIT=s,rad,rad,rad,rad/s,rad/s,rad/s,
@@ -94,7 +94,7 @@ my %addUnits = (
         Binary => 1,
     },
     AUDO => 'AudioSetting', #PH (GPMF - seen: 'WIND', fmt c)
-  # AUPT (GPMF) - seen: 'N' (fmt c)
+  # AUPT (GPMF) - seen: 'N','Y' (fmt c)
     BPOS => { #PH (Karma)
         Name => 'Controller',
         Unknown => 1,
@@ -104,9 +104,9 @@ my %addUnits = (
         %addUnits,
     },
   # BRID (GPMF) - seen: 0 (fmt B)
-  # BROD (GPMF) - seen: 'ASK' (fmt c)
+  # BROD (GPMF) - seen: 'ASK','' (fmt c)
     CASN => 'CameraSerialNumber', #PH (GPMF - seen: 'C3221324545448', fmt c)
-  # CINF (GPMF) - seen: 0x67376be7709bc8876a8baf3940908618 (fmt B)
+  # CINF (GPMF) - seen: 0x67376be7709bc8876a8baf3940908618, 0xe230988539b30cf5f016627ae8fc5395 (fmt B) (Camera INFormation?)
   # CMOD (GPMF) - seen: 12,13,17 [13 time-laps video, 17 JPEG] (fmt B)
     CYTS => { #PH (Karma)
         Name => 'CoyoteStatus',
@@ -130,13 +130,15 @@ my %addUnits = (
     DVID => { Name => 'DeviceID', Unknown => 1 }, #2 (gpmd)
   # DVNM (GPMF) seen: 'Video Global Settings' (fmt c), 'Highlights' (fmt c)
   # DVNM (gpmd) seen: 'Camera' (Hero5), 'Hero6 Black' (Hero6), 'GoPro Karma v1.0' (Karma)
-    DVNM => 'DeviceName', #PH
+    DVNM => 'DeviceName', #PH (n/c)
     DZOM => { #PH (GPMF - seen: 'Y', fmt c)
         Name => 'DigitalZoom',
         PrintConv => { N => 'No', Y => 'Yes' },
     },
   # DZST (GPMF) - seen: 0 (fmt L) (something to do with digital zoom maybe?)
-  # EISA (GPMF) - seen: 'Y','N' [N was for a time-lapse video] (fmt c)
+    EISA => { #PH (GPMF) - seen: 'Y','N', 'HS EIS' (fmt c) [N was for a time-lapse video]
+        Name => 'ElectronicImageStabilization',
+    },
   # EISE (GPMF) - seen: 'Y' (fmt c)
     EMPT => { Name => 'Empty', Unknown => 1 }, #2 (gpmd)
     ESCS => { #PH (Karma)
@@ -147,7 +149,7 @@ my %addUnits = (
         Unknown => 1,
         %addUnits,
     },
-  # EXPT (GPMF) - seen: '' (fmt c)
+  # EXPT (GPMF) - seen: '', 'AUTO' (fmt c)
     FACE => 'FaceDetected', #PH (gpmd)
     FCNM => 'FaceNumbers', #PH (gpmd) (faces counted per frame, ref 1)
     FMWR => 'FirmwareVersion', #PH (GPMF - seen: HD6.01.01.51.00, fmt c)
@@ -213,7 +215,7 @@ my %addUnits = (
         RawConv => '$val', # necessary to use scaled value instead of raw data as subdir data
         SubDirectory => { TagTable => 'Image::ExifTool::GoPro::KBAT' },
     },
-  # LINF (GPMF) - seen: LAJ7061916601668 (fmt c)
+  # LINF (GPMF) - seen: LAJ7061916601668, C3341326002180 (fmt c) (Lens INFormation?)
     LNED => { #PH (Karma)
         Name => 'LocalPositionNED',
         # UNIT=s,m,m,m,m/s,m/s,m/s
@@ -241,8 +243,9 @@ my %addUnits = (
     PHDR => 'HDRSetting', #PH (APP6 - seen: 0)
     PIMN => 'AutoISOMin', #PH (GPMF - seen: 100, fmt L)
     PIMX => 'AutoISOMax', #PH (GPMF - seen: 1600, fmt L)
-  # PRAW (APP6) - seen: 0
+  # PRAW (APP6) - seen: 0, 'N', 'Y' (fmt c)
     PRES => 'PhotoResolution', #PH (APP6 - seen: '12MP_W')
+  # PRJT (APP6) - seen: 'GPRO' (fmt F, Hero8)
     PRTN => { #PH (GPMF - seen: 'N', fmt c)
         Name => 'ProTune',
         PrintConv => {
@@ -254,6 +257,7 @@ my %addUnits = (
     PTEV => 'ExposureCompensation', #PH (GPMF - seen: '0.0', fmt c)
     PTSH => 'Sharpness', #PH (GPMF - seen: 'HIGH', fmt c)
     PTWB => 'WhiteBalance', #PH (GPMF - seen: 'AUTO', fmt c)
+  # PVUL (APP6) - seen: 'F' (fmt c, Hero8)
     RATE => 'Rate', #PH (GPMF - seen: '0_5SEC', fmt c; APP6 - seen: '4_1SEC')
     RMRK => { #2 (gpmd)
         Name => 'Comments',
@@ -291,6 +295,7 @@ my %addUnits = (
         ValueConv => '$self->Decode($val, "Latin")',
     },
   # SMTR (GPMF) - seen: 'N' (fmt c)
+  # SOFF (APP6) - seen: 0 (fmt L, Hero8)
     STMP => { #1 (gpmd)
         Name => 'TimeStamp',
         ValueConv => '$val / 1e6',
@@ -333,6 +338,7 @@ my %addUnits = (
         Unknown => 1,
         ValueConv => '$self->Decode($val, "Latin")',
     },
+  # VERS (APP6) - seen: '7 6 4' (fmt B, Hero8)
     VFOV => { #PH (GPMF - seen: 'W', fmt c)
         Name => 'FieldOfView',
         PrintConv => {
@@ -348,12 +354,13 @@ my %addUnits = (
         # UNIT=m/s,m/s,m,m/s,deg,%
         # TYPE=ffffsS
     },
-  # VLTE (GPMF) - seen: 'Y' (fmt c)
+  # VLTE (GPMF) - seen: 'Y','N' (fmt c)
     WBAL => 'ColorTemperatures', #PH (gpmd)
     WRGB => { #PH (gpmd)
         Name => 'WhiteBalanceRGB',
         Binary => 1,
     },
+  # ZFOV (APP6) - seen: 148.34 (fmt f, Hero8)
 );
 
 # GoPro GPS5 tags (ref 2) (Hero5,Hero6)
@@ -468,8 +475,8 @@ my %addUnits = (
     GROUPS => { 2 => 'Camera' },
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     NOTES => q{
-        Tags extracted from the MP4 "fdsc" timed metadata when the ExtractEmbedded
-        option is used.
+        Tags extracted from the MP4 "fdsc" timed metadata when the L<ExtractEmbedded|../ExifTool.html#ExtractEmbedded>
+        (-ee) option is used.
     },
     0x08 => { Name => 'FirmwareVersion',    Format => 'string[15]' },
     0x17 => { Name => 'SerialNumber',       Format => 'string[16]' },
@@ -613,7 +620,7 @@ sub ProcessGoPro($$$)
         next unless $size or $verbose;  # don't save empty values unless verbose
         my $format = $goProFmt{$fmt} || 'undef';
         my ($val, $i, $j, $p, @v);
-        if ($fmt eq 0x3f and defined $type) {
+        if ($fmt == 0x3f and defined $type) {
             # decode structure with format given by previous 'TYPE'
             for ($i=0; $i<$count; ++$i) {
                 my (@s, $l);
@@ -685,7 +692,7 @@ metadata from GoPro MP4 videos.
 
 =head1 AUTHOR
 
-Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
