@@ -13,9 +13,9 @@ use vars qw($VERSION);
 use Image::ExifTool;    # only for FinishTiffDump()
 use Image::ExifTool::HTML qw(EscapeHTML);
 
-$VERSION = '1.31';
+$VERSION = '1.39';
 
-sub DumpTable($$$;$$$$$);
+sub DumpTable($$$;$$$$$$);
 sub Open($$$;@);
 sub Write($@);
 
@@ -32,6 +32,7 @@ _END_PART_1_
 # Note: Don't change font-weight style because it can affect line height
 my $htmlHeader2 = <<_END_PART_2_;
 </title>
+<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
 <style type="text/css">
 <!--
 /* character style ID's */
@@ -96,7 +97,8 @@ var YTOP = 20;  // y offset when above cursor
 var safari1 = navigator.userAgent.indexOf("Safari/312.6") >= 0;
 var ie6 = navigator.userAgent.toLowerCase().indexOf('msie 6') >= 0;
 var mspan = new Array;
-var hlist, tt, tb;
+var clicked = 0;
+var hlist, tt, tb, firstOutEvt, lastInEvt;
 
 function GetElementsByClass(classname, tagname) {
   var found = new Array();
@@ -111,10 +113,23 @@ function GetElementsByClass(classname, tagname) {
       }
     }
   }
-  delete list;
   return found;
 }
 
+// click mouse
+function doClick(e)
+{
+  if (!clicked) {
+    firstOutEvt = lastInEvt = undefined;
+    high(e, 2);
+    if (hlist) clicked = 1;
+  } else {
+    clicked = 0;
+    if (firstOutEvt) high(firstOutEvt, 0);
+    if (lastInEvt) high(lastInEvt, 1);
+  }
+}
+
 // move tooltip
 function move(e)
 {
@@ -162,6 +177,12 @@ function move(e)
 
 // highlight/unhighlight text
 function high(e,on) {
+  if (on) {
+    lastInEvt = e;
+  } else {
+    if (!firstOutEvt) firstOutEvt = e;
+  }
+  if (clicked) return;
   var targ;
   if (e.target) targ = e.target;
   else if (e.srcElement) targ = e.srcElement;
@@ -171,7 +192,9 @@ function high(e,on) {
     // un-highlight current objects
     if (hlist) {
       for (var i=0; i<hlist.length; ++i) {
-        hlist[i].style.background = 'transparent';
+        for (var j=0; j<hlist[i].length; ++j) {
+          hlist[i][j].style.background = 'transparent';
+        }
       }
       hlist = null;
     }
@@ -190,20 +213,25 @@ function high(e,on) {
         }
       }
       // highlight anchor elements with the same name
-      hlist = document.getElementsByName(targ.name);
+      hlist = new Array;
+      hlist.push(document.getElementsByName(targ.name));
+      // is this an IFD pointer?
+      var pos = targ.className.indexOf('Offset_');
+      if (pos > 0) {
+        // add elements from this IFD to our highlight list
+        hlist.push(document.getElementsByClassName(targ.className.substr(pos+7)));
+      }
       // use class name to highlight span elements if necessary
       for (var i=0; i<mspan.length; ++i) {
         if (mspan[i] != targ.name) continue;
-        var slist = GetElementsByClass(targ.name, 'span');
-        // add elements from hlist collection to our array
-        for (var j=0; j<hlist.length; ++j) {
-            slist[slist.length] = hlist[j];
-        }
-        hlist = slist;
+        // add these span elements to our highlight list
+        hlist.push(GetElementsByClass(targ.name, 'span'));
         break;
       }
-      for (var j=0; j<hlist.length; ++j) {
-        hlist[j].style.background = '#ffcc99';
+      for (var i=0; i<hlist.length; ++i) {
+        for (var j=0; j<hlist[i].length; ++j) {
+          hlist[i][j].style.background = on == 2 ? '#ffbbbb' : '#ffcc99';
+        }
       }
     }
   }
@@ -219,7 +247,7 @@ Enable JavaScript for active highlighting and information tool tips!
 <table class=dump cellspacing=0 cellpadding=2>
 <tr><td valign='top'><pre>];
 
-my $preMouse = q(<pre onmouseover="high(event,1)" onmouseout="high(event,0)" onmousemove="move(event)">);
+my $preMouse = q(<pre onmouseover="high(event,1)" onmouseout="high(event,0)" onmousemove="move(event)" onmousedown="doClick(event)">);
 
 #------------------------------------------------------------------------------
 # New - create new HtmlDump object
@@ -236,7 +264,7 @@ sub new
 # Add information to dump
 # Inputs: 0) HTML dump hash ref, 1) absolute offset in file, 2) data size,
 #         3) comment string, 4) tool tip (or SAME to use previous tip),
-#         5) bit flags (see below)
+#         5) bit flags (see below), 6) IFD name
 # Bits: 0x01 - print at start of line
 #       0x02 - print red address
 #       0x04 - maker notes data ('M'-class span)
@@ -244,9 +272,9 @@ sub new
 #       0x10 - allow double references
 #       0x100 - (reserved)
 # Notes: Block will be shown in 'unused' color if comment string begins with '['
-sub Add($$$$;$)
+sub Add($$$$;$$)
 {
-    my ($self, $start, $size, $msg, $tip, $flag) = @_;
+    my ($self, $start, $size, $msg, $tip, $flag, $ifd) = @_;
     my $block = $$self{Block};
     $$block{$start} or $$block{$start} = [ ];
     my $htip;
@@ -263,7 +291,7 @@ sub Add($$$$;$)
         $htip .= "<br>($size bytes)" unless $htip =~ /<br>Size:/;
         ++$self->{TipNum};
     }
-    push @{$$block{$start}}, [ $size, $msg, $htip, $flag, $self->{TipNum} ];
+    push @{$$block{$start}}, [ $size, $msg, $htip, $flag, $self->{TipNum}, $ifd ];
 }
 
 #------------------------------------------------------------------------------
@@ -351,7 +379,7 @@ sub Print($$;$$$$$)
         }
         my $parms;
         foreach $parms (@$parmList) {
-            my ($len, $msg, $tip, $flag, $tipNum) = @$parms;
+            my ($len, $msg, $tip, $flag, $tipNum, $ifd) = @$parms;
             next unless $len > 0;
             $flag = 0 unless defined $flag;
             # generate same name for all blocks indexed by this tooltip
@@ -367,8 +395,10 @@ sub Print($$;$$$$$)
                 ++$index;
             }
             if ($flag & 0x14) {
+                my $class = $flag & 0x04 ? "$name M" : $name;
+                $class .= " $ifd" if $ifd;
                 my %bkg = (
-                    Class => $flag & 0x04 ? "$name M" : $name,
+                    Class => $class,
                     Start => $start - $dataPos,
                     End   => $start - $dataPos + $len,
                 );
@@ -383,7 +413,9 @@ sub Print($$;$$$$$)
             for ($try=0; $try<2; ++$try) {
                 $end = $start + $len;
                 # only load as much of the block as we are going to dump
-                my $size = ($len > $limit) ? $limit / 2 : $len;
+                # (read 32 more bytes than necessary just in case there
+                # is only one skipped line that we decide to print)
+                my $size = ($len > $limit + 32) ? $limit / 2 + 16 : $len;
                 if ($start >= $dataPos and $end <= $dataEnd) {
                     $buff = substr($$dataPt, $start-$dataPos, $size);
                     if ($len != $size) {
@@ -426,7 +458,7 @@ sub Print($$;$$$$$)
                 $flag |= 0x100 unless $flag & 0x01 or $nextFlag & 0x01;
             }
             $self->DumpTable($start-$dataPos, \$buff, $msg, $name,
-                             $flag, $len, $pos-$dataPos);
+                             $flag, $len, $pos-$dataPos, $ifd);
             undef $buff;
             $pos = $end if $pos < $end;
         }
@@ -556,10 +588,10 @@ sub Open($$$;@)
 # Dump a block of data in HTML table form
 # Inputs: 0) HtmlDump object ref, 1) data position, 2) block pointer,
 #         3) message, 4) object name, 5) flag, 6) full block length (actual
-#         data may be shorter), 7) data end position
-sub DumpTable($$$;$$$$$)
+#         data may be shorter), 7) data end position, 8) IFD name
+sub DumpTable($$$;$$$$$$)
 {
-    my ($self, $pos, $blockPt, $msg, $name, $flag, $len, $endPos) = @_;
+    my ($self, $pos, $blockPt, $msg, $name, $flag, $len, $endPos, $ifd) = @_;
     $len = length $$blockPt unless defined $len;
     $endPos = 0 unless $endPos;
     my ($f0, $dblRef, $id);
@@ -586,7 +618,8 @@ sub DumpTable($$$;$$$$$)
             }
             ++$id unless $dblRef;
         }
-        $name = "<a name=$name class=$id>";
+        my $class = $ifd ? "'$id $ifd'" : $id;
+        $name = "<a name=$name class=$class>";
         $msg and $msg = "$name$msg</a>";
     } else {
         $name = '';
@@ -651,7 +684,8 @@ sub DumpTable($$$;$$$$$)
         if ($dblRef and $p >= $endPos) {
             $dblRef = 0;
             ++$id;
-            $name =~ s/class=\w\b/class=$id/;
+            my $class = $ifd ? "'$id $ifd'" : $id;
+            $name =~ s/class=\w\b/class=$class/;
             $f0 = '';
             $self->Open('fgd', $f0, 0);
         }
@@ -721,7 +755,7 @@ sub DumpTable($$$;$$$$$)
 #  is only compiled when needed)
 sub FinishTiffDump($$$)
 {
-    my ($self, $exifTool, $size) = @_;
+    my ($self, $et, $size) = @_;
     my ($tag, $key, $start, $blockInfo, $i);
 
     # list of all indirectly referenced TIFF data tags
@@ -737,21 +771,22 @@ sub FinishTiffDump($$$)
         AlphaOffset       => 'AlphaByteCount',
         MPImageStart      => 'MPImageLength',
         IDCPreviewStart   => 'IDCPreviewLength',
+        SamsungRawPointersOffset => 'SamsungRawPointersLength',
     );
 
     # add TIFF data to html dump
     foreach $tag (keys %offsetPair) {
-        my $info = $exifTool->GetInfo($tag);
+        my $info = $et->GetInfo($tag);
         next unless %$info;
         # Panasonic hack: StripOffsets is not valid for Panasonic RW2 files,
         # and StripRowBytes is not valid for some RAW images
-        if ($tag eq 'StripOffsets' and $exifTool->{TAG_INFO}{$tag}{PanasonicHack}) {
+        if ($tag eq 'StripOffsets' and $$et{TAG_INFO}{$tag}{PanasonicHack}) {
             # use RawDataOffset instead if available since it is valid in RW2
-            my $info2 = $exifTool->GetInfo('RawDataOffset');
+            my $info2 = $et->GetInfo('RawDataOffset');
             $info2 = $info unless %$info2;
             my @keys = keys %$info2;
             my $offset = $$info2{$keys[0]};
-            my $raf = $$exifTool{RAF};
+            my $raf = $$et{RAF};
             # ignore StripByteCounts and assume raw data runs to the end of file
             if (@keys == 1 and $offset =~ /^\d+$/ and $raf) {
                 my $pos = $raf->Tell();
@@ -767,8 +802,8 @@ sub FinishTiffDump($$$)
         # loop through all offsets tags
         foreach $key (keys %$info) {
             my $name = Image::ExifTool::GetTagName($key);
-            my $grp1 = $exifTool->GetGroup($key, 1);
-            my $info2 = $exifTool->GetInfo($offsetPair{$tag}, { Group1 => $grp1 });
+            my $grp1 = $et->GetGroup($key, 1);
+            my $info2 = $et->GetInfo($offsetPair{$tag}, { Group1 => $grp1 });
             my $key2 = $offsetPair{$tag};
             $key2 .= $1 if $key =~ /( .*)/; # use same instance number as $tag
             next unless $$info2{$key2};
@@ -799,7 +834,7 @@ sub FinishTiffDump($$$)
                         next;
                     }
                 }
-                my $msg = $exifTool->GetGroup($key, 1) . ':' . $tag;
+                my $msg = $et->GetGroup($key, 1) . ':' . $tag;
                 $msg =~ s/(Offsets?|Start)$/ /;
                 if ($num > 1) {
                     $msg .= "$li-" if $li != $i;
@@ -823,7 +858,7 @@ sub FinishTiffDump($$$)
         }
     }
     my $diff = $size - $last;
-    if ($diff > 0 and ($last or $exifTool->Options('Unknown'))) {
+    if ($diff > 0 and ($last or $et->Options('Unknown'))) {
         if ($diff > 1 or $size & 0x01) {
             $self->Add($last, $diff, "[unknown data]", "Size: $diff bytes", 0x08);
         } else {
@@ -882,7 +917,7 @@ page.
 
 =head1 AUTHOR
 
-Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
