@@ -16,7 +16,7 @@ use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::XMP;
 
-$VERSION = '1.23';
+$VERSION = '1.24';
 
 sub RecoverTruncatedIPTC($$$);
 sub ListToString($);
@@ -460,7 +460,8 @@ my %sKeywordStruct;
     NOTES => q{
         Image region metadata defined by the MWG 2.0 specification.  These tags
         may be accessed without the need to load the MWG Composite tags above.  See
-        L<http://www.metadataworkinggroup.org/> for the official specification.
+        L<https://web.archive.org/web/20181006115950/http://www.metadataworkinggroup.org/specs/>
+        for the official specification.
     },
     Regions => {
         Name => 'RegionInfo',
@@ -488,8 +489,9 @@ my %sKeywordStruct;
         Hierarchical keywords metadata defined by the MWG 2.0 specification. 
         ExifTool unrolls keyword structures to an arbitrary depth of 6 to allow
         individual levels to be accessed with different tag names, and to avoid
-        infinite recursion.  See L<http://www.metadataworkinggroup.org/> for the
-        official specification.
+        infinite recursion.  See
+        L<https://web.archive.org/web/20181006115950/http://www.metadataworkinggroup.org/specs/>
+        for the official specification.
     },
     # arbitrarily define only the first 6 levels of the keyword hierarchy
     Keywords => {
@@ -527,7 +529,8 @@ my %sKeywordStruct;
     NAMESPACE => 'mwg-coll',
     NOTES => q{
         Collections metadata defined by the MWG 2.0 specification.  See
-        L<http://www.metadataworkinggroup.org/> for the official specification.
+        L<https://web.archive.org/web/20181006115950/http://www.metadataworkinggroup.org/specs/>
+        for the official specification.
     },
     Collections => {
         FlatName => '',
@@ -626,6 +629,14 @@ sub OverwriteStringList($$$$)
     local $_;
     my ($et, $nvHash, $val, $newValuePt) = @_;
     my (@new, $delIndex);
+    my $writeMode = $et->Options('WriteMode');
+    if ($writeMode ne 'wcg') {
+        if (defined $val) {
+            $writeMode =~ /w/i or return 0;
+        } else {
+            $writeMode =~ /c/i or return 0;
+        }
+    }
     if ($$nvHash{DelValue} and defined $val) {
         # preserve specified old values
         my $old = StringToList($val, $et);
